#include<autoconf.h>
#include<board_config.h>

#ifdef CONFIG_BOOT_TOOL
#define HEAD_LENGTH 0
#else
#define HEAD_LENGTH 24
#endif

#define STAGE2_TEXT       *(.text*)
#define STAGE2_DATA       *(.data*)
#define STAGE2_RODATA     *(.rodata*)
#define STAGE2_XIP_TEXT   *(.xip.text)
#define STAGE2_XIP_RODATA *(.xip.rodata*)

# define RUN_AT_STAGE2_TEXT (tmp_objs/vic.o \
                            tmp_objs/clock_board.o \
                            tmp_objs/flash_spi.o \
                            tmp_objs/flash.o  \
                            tmp_objs/pmu_ctrl.o \
                            tmp_objs/ldo.o \
                            tmp_objs/clock.o \
                            tmp_objs/start.o \
                            tmp_objs/spl_start.o \
                            tmp_objs/system.o \
                            tmp_objs/trap_c.o \
                            tmp_objs/vectors.o \
                            tmp_objs/systick_counter.o \
                            tmp_objs/uni_counter.o \
                            tmp_objs/uni_dcache.o \
                            tmp_objs/memcpy.o \
                            tmp_objs/memcpy_fast.o \
                            tmp_objs/memset.o \
                            tmp_objs/memset_fast.o \
                            tmp_objs/ef_exp.o \
                            tmp_objs/wf_exp.o \
                            tmp_objs/uni_icache.o)

/* LMA */
#ifdef CONFIG_MCU_ENABLE_JTAG_DEBUG
# define STAGE1_SRAM_TEXT_LMA
# define STAGE1_SRAM_DATA_LMA
# define STAGE2_XIP_TEXT_LMA
# define STAGE2_SRAM_TEXT_LMA
# define STAGE2_SRAM_DATA_LMA
# define AUDIO_IN_LMA
# define NPU_LMA
#else
# define STAGE1_SRAM_TEXT_LMA  AT(0)
# define STAGE1_SRAM_DATA_LMA  AT(SIZEOF(.stage1_sram_text))
# ifdef CONFIG_MCU_ENABLE_XIP
#  define STAGE2_XIP_TEXT_LMA  AT(LENGTH(stage1_dram)-HEAD_LENGTH)
#  define STAGE2_SRAM_TEXT_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_xip_text))
#  define STAGE2_SRAM_DATA_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_xip_text)+SIZEOF(.stage2_sram_text))
#  define AUDIO_OUT_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_xip_text)+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data))
#  define AUDIO_IN_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_xip_text)+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data)+SIZEOF(.audio_out_section))
#  define NPU_LMA      AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_xip_text)+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data)+SIZEOF(.audio_out_section)+SIZEOF(.audio_in_section))
# else
#  define STAGE2_XIP_TEXT_LMA
#  define STAGE2_SRAM_TEXT_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH)
#  define STAGE2_SRAM_DATA_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_sram_text))
#  define AUDIO_OUT_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data))
#  define AUDIO_IN_LMA AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data)+SIZEOF(.audio_out_section))
#  define NPU_LMA      AT(LENGTH(stage1_dram)-HEAD_LENGTH+SIZEOF(.stage2_sram_text)+SIZEOF(.stage2_sram_data)+SIZEOF(.audio_in_section)+SIZEOF(.audio_out_section))
# endif
#endif

ENTRY(spl_reset_handler);

MEMORY
{
    stage1_iram : ORIGIN = CONFIG_STAGE1_IRAM_BASE, LENGTH = CONFIG_STAGE1_IRAM_SIZE
    stage1_dram : ORIGIN = CONFIG_STAGE1_DRAM_BASE, LENGTH = CONFIG_STAGE1_DRAM_SIZE
    stage2_iram : ORIGIN = CONFIG_STAGE2_IRAM_BASE, LENGTH = CONFIG_STAGE2_IRAM_SIZE
#ifdef CONFIG_MCU_ENABLE_XIP
    stage2_xip  : ORIGIN = CONFIG_STAGE2_XIP_BASE,  LENGTH = CONFIG_STAGE2_XIP_SIZE
#endif
    stage2_dram : ORIGIN = CONFIG_STAGE2_DRAM_BASE, LENGTH = CONFIG_STAGE2_DRAM_SIZE
    audio_out_dram : ORIGIN = 0x20030000 - CONFIG_AUDIO_OUT_BUFFER_SIZE, LENGTH = CONFIG_AUDIO_OUT_BUFFER_SIZE
    audio_in_dram : ORIGIN = 0x20030000, LENGTH = 0x4000       /* audio_in 固定从0x20030000地址开始 */
}

SECTIONS
{
    .stage1_sram_text : STAGE1_SRAM_TEXT_LMA
    {
        . = ALIGN(4);
        _stage1_sram_start_text_ = .;
        tmp_objs/spl_start.o(.spl.vectors*)
        tmp_objs/spl_start.o(.text*)
        tmp_objs/spl.o(.text*)
        tmp_objs/spl_clk.o(.text*)
        tmp_objs/spl_spinor.o(.text*)
        tmp_objs/spl_dw_dmac_ahb.o(.text*)
        tmp_objs/spl_i2c.o(.text*)
        tmp_objs/spl_counter.o(.text*)
        tmp_objs/spl_uart.o(.text*)
        tmp_objs/boot_board.o(.text*)

        tmp_objs/spl_start.o(.rodata*)
        tmp_objs/spl.o(.rodata*)
        tmp_objs/spl_clk.o(.rodata*)
        tmp_objs/spl_spinor.o(.rodata*)
        tmp_objs/spl_dw_dmac_ahb.o(.rodata*)
        tmp_objs/spl_i2c.o(.text*)
        tmp_objs/spl_counter.o(.text*)
        tmp_objs/spl_uart.o(.text*)
        tmp_objs/boot_board.o(.rodata*)

        . = ALIGN(4);
        _stage1_sram_end_text = .;
    } > stage1_iram

    .dummy_for_stage1_sram_text_place_hold :
    {
        . += SIZEOF(.stage1_sram_text);
    } > stage1_dram

    .stage1_sram_data ALIGN(4) : STAGE1_SRAM_DATA_LMA
    {
        . = ALIGN(4);
        _stage1_sram_start_data_ = .;
        tmp_objs/spl_start.o(.data*)
        tmp_objs/spl.o(.data*)
        tmp_objs/spl_clk.o(.data*)
        tmp_objs/spl_spinor.o(.data*)
        tmp_objs/spl_dw_dmac_ahb.o(.data*)
        tmp_objs/spl_i2c.o(.text*)
        tmp_objs/spl_counter.o(.text*)
        tmp_objs/spl_uart.o(.text*)
        tmp_objs/boot_board.o(.data*)

        . = ALIGN(4);
        _stage1_sram_end_data_ = .;
    } > stage1_dram

    . = ALIGN(2048);

#ifdef CONFIG_MCU_ENABLE_XIP
    .stage2_xip_text : STAGE2_XIP_TEXT_LMA
    {
        _stage2_xip_start_text_ = .;
        STAGE2_XIP_TEXT
        STAGE2_XIP_RODATA
# ifdef CONFIG_MCU_DEFAULT_TEXT_IN_FLASH
        *(EXCLUDE_FILE RUN_AT_STAGE2_TEXT .text*)
        *(EXCLUDE_FILE RUN_AT_STAGE2_TEXT .rodata*)
#  ifdef CONFIG_UNI_COUNTER
        tmp_objs/uni_counter.o(.rodata*)
#  endif
        tmp_objs/uni_icache.o(.rodata*)
        tmp_objs/uni_dcache.o(.rodata*)
        tmp_objs/vic.o(.rodata*)
        tmp_objs/flash.o(.rodata*)
        tmp_objs/flash_spi.o(.rodata*)
        tmp_objs/ldo.o(.rodata*)
# endif

# ifdef CONFIG_NPU_RUN_IN_FLASH
        . = ALIGN(4);
        KEEP(*(.cmd*))
        . = ALIGN(4);
        KEEP(*(.weight*))
        . = ALIGN(4);
        KEEP(*(.npu_section*))
# endif
        . = ALIGN(4);
        _stage2_xip_end_text_ = .;
    } > stage2_xip
#endif /* CONFIG_MCU_ENABLE_XIP */

    .stage2_sram_text : STAGE2_SRAM_TEXT_LMA
    {
        _stage2_sram_start_text_ = .;
        tmp_objs/start.o(.vectors*)
        tmp_objs/start.o(.text*)
        tmp_objs/spl_start.o(.rodata*)
#ifdef CONFIG_MCU_DEFAULT_TEXT_IN_FLASH
        tmp_objs/system.o(.text*)
        tmp_objs/system.o(.rodata*)
        tmp_objs/trap_c.o(.text*)
        tmp_objs/trap_c.o(.rodata*)
        tmp_objs/vectors.o(.text*)
        tmp_objs/vectors.o(.rodata*)
        tmp_objs/flash_spi.o(.text*)
        tmp_objs/ldo.o(.text*)
        tmp_objs/flash.o(.text*)
        tmp_objs/pmu_ctrl.o(.text*)
        tmp_objs/pmu_ctrl.o(.rodata*)
        tmp_objs/clock.o(.text*)
        tmp_objs/clock.o(.rodata*)
        tmp_objs/vic.o(.text*)
        tmp_objs/uni_dcache.o(.text*)
        tmp_objs/uni_icache.o(.text*)
        tmp_objs/memcpy.o(.text*)
        tmp_objs/memcpy_fast.o(.text*)
# ifdef CONFIG_LVP_DISABLE_XIP_WHILE_CODE_RUN_AT_SRAM
        tmp_objs/memset.o(.text*)
        tmp_objs/memset_fast.o(.text*)
        tmp_objs/ef_exp.o(.text*)
        tmp_objs/ef_exp.o(.rodata*)
        tmp_objs/wf_exp.o(.text*)
        tmp_objs/wf_exp.o(.rodata*)
# endif
# ifdef CONFIG_UNI_COUNTER
        tmp_objs/uni_counter.o(.text*)
# endif
        tmp_objs/clock_board.o(.text*)
        tmp_objs/clock_board.o(.rodata*)
#else
        STAGE2_TEXT
        STAGE2_RODATA
#endif
        KEEP(*(.sram_text*))
        . = ALIGN(4);
        _stage2_sram_end_text_ = .;
    } > stage2_iram

    .dummy_for_stage2_sram_text_place_hold :
    {
        . += SIZEOF(.stage2_sram_text);
    } > stage2_dram

    .stage2_sram_data ALIGN(4) : STAGE2_SRAM_DATA_LMA
    {
        _stage2_sram_start_data_ = .;
        tmp_objs/start.o(.data*)
        STAGE2_DATA
        _stage2_sram_end_data_ = .;
    } > stage2_dram

    . = ALIGN(4);
    .bss :
    {
        _start_bss_ = .;
        *(.bss)
        *(COMMON)
        . = ALIGN(4);
        _end_bss_ = .;
    } > stage2_dram
    .dummy_for_stage2_stack_hold :
    {
        _start_stack_section_ = .;
        . += CONFIG_MCU_MAIN_STACK_SIZE * 1024;
        _end_stack_section_ = .;
    } > stage2_dram

    .audio_out_dummy :
    {
        . += .;
    } > audio_out_dram

    . = ALIGN(16);
    .audio_out_section ALIGN(16): AUDIO_OUT_LMA
    {
        _start_audio_out_section_ = .;
        KEEP(*(.audio_out*))
        _end_audio_out_section_ = .;
    } > audio_out_dram

    .audio_in_dummy :
    {
        . += .;
    } > audio_in_dram

    . = ALIGN(4);
    .audio_in_section ALIGN(4): AUDIO_IN_LMA
    {
        _start_audio_in_section_ = .;
        KEEP(*(.audio_in*))
        _end_audio_in_section_ = .;
    } > audio_in_dram

#ifndef CONFIG_NPU_RUN_IN_FLASH

# ifdef CONFIG_LVP_ENABLE_KEYWORD_RECOGNITION
    . = ALIGN(4);
    .npu_section ALIGN(4): AT(0x18000-HEAD_LENGTH)
    {
        _start_npu_section_ = .;
        KEEP(*(.cmd*))
        . = ALIGN(4);
        KEEP(*(.weight*))
        . = ALIGN(4);
        _end_npu_section_ = .;
    }
# endif

#endif /* CONFIG_MCU_ENABLE_XIP */

}
